#include "testApp.h"

#define HOST "128.237.239.169" // 
#define PORT 12345
#define RECONNECT_TIME 400

#define FLASH_DELAY 50

double flash_last = 0;

//ofxBox2dCircle theMouse;

//--------------------------------------------------------------
void testApp::setup(){
	ofBackground(0, 0, 0);
	ofEnableSmoothing();
	context.setupUsingXMLFile();//ofToDataPath(""config/SamplesConfig.xml"));
	context.enableLogging(OF_LOG_SILENT);
	depth.setup(&context);
	rgbimage.setup(&context);
	user.setup(&context, &depth);
	
	// SOCKET
	msgTx	= "";
	msgRx	= "";
	lastClient = "";
	
	//are we connected to the server - if this fails we
	//will check every few seconds to see if the server exists
	weConnected = tcpClient.setup(HOST, PORT);
	connectTime = 0;
	deltaTime = 0;
	
	//tcpClient.setVerbose(true);
	
	ofSetFrameRate(60);
	
	// box2d
	bDrawLines  = false;
	bMouseForce = false;
	
	box2d.init();
	box2d.setGravity(0, 12.0);
	box2d.createFloor();
	box2d.checkBounds(true);
	box2d.setFPS(60.0);
	
//	theMouse.setPhysics(3.0, 0.53, 0);
//	theMouse.enableGravity(false);
//	theMouse.setup(box2d.getWorld(), mouseX, mouseY, 40);
	
	// lets draw a simple landscape
	/**/
	ofPoint p(0, 480);
	lineStrip.setWorld(box2d.getWorld());
	lineStrip.clear();
	lineStrip.addPoint(640,480);
	lineStrip.createShape();
	/**/
}

void testApp::parseString() {
	// clientid~uniqid~x~y~action~radius~
	// 0        1      2 3 4      5
	int i;
	string tmp="";
	int count = 0;
	int l = msgRx.length();
	string clientcode;
	int x;
	int y;
	int radius;
	int action;
	ofPoint tmpPoint;
	client tmpClient;
	for (i=0;i<l;++i) {
		if (msgRx[i] != '[') {
			if (msgRx[i] != '~') {
				tmp += msgRx[i];
			} else {
				switch (count) {
					case 0:
						break;
					case 1:
						clientcode = tmp.c_str();
						lastClient = clientcode;
						break;
					case 2:
						x = atoi(tmp.c_str());
						break;
					case 3:
						y = atoi(tmp.c_str());
						break;
					case 4:
						action = atoi(tmp.c_str());
						break;
					case 5:
						radius = atoi(tmp.c_str());
						break;
					default:
						break;
				}
				tmpPoint.x = x;
				tmpPoint.y = y;
				tmpClient.pos = tmpPoint;
				tmpClient.action = action;
				tmpClient.radius = radius;
				if (clients[clientcode].color==0)
					tmpClient.color = ofRandom(0, 0xffffff);
				clients[clientcode] = tmpClient;
				count++;
				tmp = "";
			}
		} else {
			break;
		}
		
	}
}

//--------------------------------------------------------------
void testApp::update(){
	box2d.update();
	context.update();
	user.update();
	updateSocket();
}

void testApp::updateSocket() {
	/**
	 /deviceID/userid/limb
	/**/
	//we are connected - lets send our text and check what we get back
	if (weConnected) {
		string str = tcpClient.receive();
		if ( str.length() > 0 ) {
			msgRx =  str;
			parseString();
		} else if (!tcpClient.isConnected()) {
			weConnected = false;
		}
	} else {
		//if we are not connected lets try and reconnect every 5 seconds
		deltaTime = ofGetElapsedTimeMillis() - connectTime;
		
		if ( deltaTime > 5000 ) {
			weConnected = tcpClient.setup(HOST, PORT);
			connectTime = ofGetElapsedTimeMillis();
		}
	}
}	

void testApp::drawBox2D () {
	// box2d
	for(int i=0; i<circles.size(); i++) {
		ofPoint p = circles[i].getPosition();
		if (p.x<-60||p.x>700||p.y>580) {
			circles[i].destroyShape();
		} else {
			circles[i].draw();
			ofSetColor(255, 0, 0);
			ofFill();
			ofCircle(p.x, p.y, circles[i].getRadius()+1);
		}

	}
	for(int i=0; i<polygons.size(); i++) {
		polygons[i].draw();
	}
	for(int i=0; i<boxes.size(); i++) {
		boxes[i].draw();
	}
	for(int i=0; i<lines.size(); i++) {
		lines[i].draw();
	}
	for(int i=0; i<customParticles.size(); i++) {
		customParticles[i].draw();
	}
	
	
	//theMouse.draw();
	lineStrip.draw();
	box2d.draw();
	
	px = mouseX;
	py = mouseY;
	
	/**
	string info = "";
	info += "Press [s] to draw a line strip ["+ofToString(bDrawLines)+"]\n"; 
	info += "Press [f] to toggle Mouse Force ["+ofToString(bMouseForce)+"]\n"; 
	info += "Press [c] for circles\n";
	info += "Press [b] for blocks\n";
	info += "Press [z] for custom particle\n";
	info += "Total Bodies: "+ofToString(box2d.getBodyCount())+"\n";
	info += "Total Joints: "+ofToString(box2d.getJointCount())+"\n\n";
	info += "FPS: "+ofToString(ofGetFrameRate())+"\n";
	ofSetColor(255, 255, 255);
	ofDrawBitmapString(info, 30, 150);
	/**/
}

//--------------------------------------------------------------
void testApp::draw(){
	//depth.draw(0,0,160,120);
	/**
	rgbimage.draw(0, 0, 640, 480);
	flippedimage.grabScreen(0, 0, 640, 480);
	flippedimage.mirror(true, false);
	flippedimage.draw(0, 0);
	/**/
	drawBox2D();
	
	// mga
	drawUsers();
	drawCursor();
}

void testApp::drawUsers() {
	int i;
	int nTrackedUsers = user.getTrackedUsers().size();
	ofxTrackedUser *tracked;
	for (i=0; i<nTrackedUsers; i++) {
		// get the user
		tracked = user.getTrackedUser(i);
		if(tracked != NULL) {
			// there is a skeleton
			// draw it
//			tracked->debugDraw();
			drawUserDots(i,tracked);
			// decompose it
			// put in the server string
		}
	}
	skeletonsToFlash();
}

void testApp::drawCursor() {
	/**
	 for(map<key, value>::const_iterator i = theMap.begin(); i != theMap.end(); ++i) {
	 key k = i->first;
	 // ...
	 }
	/**
	ofSetRectMode(OF_RECTMODE_CENTER);
	ofDrawBitmapString("Debug:\n", 10, 500);
	for(map<string, client>::const_iterator i = clients.begin(); i != clients.end(); i++) {
		client cl = i->second;
		ofSetColor(255, 0, 0);
		ofCircle(cl.pos.x, cl.pos.y, 5);
		ofDrawBitmapString(ofToString(cl.pos.x) + ":" + ofToString(cl.pos.y) + "\n", 10, 510);
	}
	//it = clients.find(lastClient);
	//client cl = it->second;
	ofSetRectMode(OF_RECTMODE_CORNER);
	/**/
	for(map<string, client>::const_iterator i = clients.begin(); i != clients.end(); i++) {
		client cl = i->second;
		if (!cl.drawn && cl.action==2) {
			cl.drawn = true;
			ofxBox2dCircle circle;
			circle.setPhysics(0.1, 0.933, 0);
			circle.setup(box2d.getWorld(), cl.pos.x, cl.pos.y, cl.radius/2);
			circles.push_back(circle);
		}
		ofSetColor(255, 0, 255);
		ofCircle(cl.pos.x, cl.pos.y, cl.radius/2);
	}
}

void testApp::drawUserDots(int i, ofxTrackedUser *skeleton) {
	string sid = "user" + ofToString(i);
	drawLimb(sid,"neck",skeleton->neck);
	
	// left arm + shoulder
	drawLimb(sid,"left_shoulder",skeleton->left_shoulder);
	drawLimb(sid,"left_upper_arm",skeleton->left_upper_arm);
	drawLimb(sid,"left_lower_arm",skeleton->left_lower_arm);
	
	// right arm + shoulder
	drawLimb(sid,"right_shoulder",skeleton->right_shoulder);
	drawLimb(sid,"right_upper_arm",skeleton->right_upper_arm);
	drawLimb(sid,"right_lower_arm",skeleton->right_lower_arm);
	
	// upper torso
	drawLimb(sid,"left_upper_torso",skeleton->left_upper_torso);
	drawLimb(sid,"right_upper_torso",skeleton->right_upper_torso);
	
	// left lower torso + leg
	drawLimb(sid,"left_lower_torso",skeleton->left_lower_torso);
	drawLimb(sid,"left_upper_leg",skeleton->left_upper_leg);
	drawLimb(sid,"left_lower_leg",skeleton->left_lower_leg);
	
	// right lower torso + leg
	drawLimb(sid,"right_lower_torso",skeleton->right_lower_torso);
	drawLimb(sid,"right_upper_leg",skeleton->right_upper_leg);
	drawLimb(sid,"right_lower_leg",skeleton->right_lower_leg);
	
	drawLimb(sid,"hip",skeleton->hip);
	if (!skeletonsDrawn[sid]) skeletonsDrawn[sid] = true;
}

void testApp::drawLimb(string uid, string l, ofxLimb &limb) {
	/**
	/**/
	// box2d skeleton
	int rb = 32-ABS(ofMap(limb.begin.z, 0, 2048, 0, 26, true));
	int re = 32-ABS(ofMap(limb.end.z, 0, 2048, 0, 26, true));
	if (!skeletonsDrawn[uid]) {
		// initial setup
		userDots[uid+l+"b"].setPhysics(3.0, 1.53, 1.1);
		userDots[uid+l+"b"].enableGravity(false);
		userDots[uid+l+"b"].setup(box2d.getWorld(), limb.begin.x, limb.begin.y, rb);
		userDots[uid+l+"e"].setPhysics(3.0, 1.53, 1.1);
		userDots[uid+l+"e"].enableGravity(false);
		userDots[uid+l+"e"].setup(box2d.getWorld(), limb.end.x, limb.end.y, re);
		//userJoints[l].setWorld(box2d.getWorld());
		//userJoints[l].addJoint(userDots[uid+l+"b"].body, userDots[uid+l+"e"].body, 3.0, 0.5);
	} else {
		userDots[uid+l+"b"].setRadius(rb);
		userDots[uid+l+"b"].setPosition(limb.begin.x, limb.begin.y);
		userDots[uid+l+"e"].setRadius(re);
		userDots[uid+l+"e"].setPosition(limb.end.x, limb.end.y);
	}
	userDots[uid+l+"b"].draw();
	userDots[uid+l+"e"].draw();
	//userJoints[l].draw();
	/**/
	// visual
	ofSetColor(0, 0, 0);
	ofSetLineWidth(0);
	ofFill();
	ofCircle(limb.begin.x, limb.begin.y, rb*1.33);
	ofCircle(limb.end.x, limb.end.y, re*1.33);
	ofSetColor(255, 255, 255);
	ofCircle(limb.begin.x, limb.begin.y, rb);
	ofCircle(limb.end.x, limb.end.y, re);
	if (ofRandom(0, 100)>90) {
		ofSetLineWidth(2);
		ofLine(limb.begin.x, limb.begin.y, limb.end.x, limb.end.y);
	}
	/**/
}

void testApp::skeletonsToFlash() {
	int nTrackedUsers = user.getTrackedUsers().size();
	if (nTrackedUsers>0) {
		ofxTrackedUser *skeleton;
		/**/
		string m = "";
		int i;
		for (i=0; i<nTrackedUsers; i++) {
			// get the user
			skeleton = user.getTrackedUser(i);
			m += limbToFlash(i, "neck", skeleton->neck);
			
			// left arm + shoulder
			m += "|" + limbToFlash(i, "left_shoulder", skeleton->left_shoulder);
			m += "|" + limbToFlash(i, "left_upper_arm", skeleton->left_upper_arm);
			m += "|" + limbToFlash(i, "left_lower_arm", skeleton->left_lower_arm);
			
			// right arm + shoulder
			m += "|" + limbToFlash(i, "right_shoulder", skeleton->right_shoulder);
			m += "|" + limbToFlash(i, "right_upper_arm", skeleton->right_upper_arm);
			m += "|" + limbToFlash(i, "right_lower_arm", skeleton->right_lower_arm);
			
			// torso
			m += "|" + limbToFlash(i, "left_upper_torso", skeleton->left_upper_torso);
			m += "|" + limbToFlash(i, "right_upper_torso", skeleton->right_upper_torso);
			
			// left lower torso + leg
			m += "|" + limbToFlash(i, "left_lower_torso", skeleton->left_lower_torso);
			m += "|" + limbToFlash(i, "left_upper_leg", skeleton->left_upper_leg);
			m += "|" + limbToFlash(i, "left_lower_leg", skeleton->left_lower_leg);
			
			// right lower torso + leg
			m += "|" + limbToFlash(i, "right_lower_torso", skeleton->right_lower_torso);
			m += "|" + limbToFlash(i, "right_upper_leg", skeleton->right_upper_leg);
			m += "|" + limbToFlash(i, "right_lower_leg", skeleton->right_lower_leg);
			
			m += "|" + limbToFlash(i, "hip", skeleton->hip);
			m += "}";
		}
		
		if (ofGetElapsedTimeMillis() - flash_last > FLASH_DELAY) {
			flash_last = ofGetElapsedTimeMillis();
			tcpClient.send(m);
		}
	}
	/**/
}

string testApp::limbToFlash(int i, string address, ofxLimb &limb) {
	/**/
	string m = "";
	m += ofToString( limb.begin.x );
	m += ":" + ofToString( limb.begin.y );
	m += ":" + ofToString( limb.begin.z );
	m += ":" + ofToString( limb.end.x );
	m += ":" + ofToString( limb.end.y );
	m += ":" + ofToString( limb.end.z );
	m += ":";
	return m;
	/**/
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
	if(key == 'c') {
		float r = ofRandom(4, 20);		// a random radius 4px - 20px
		ofxBox2dCircle circle;
		circle.setPhysics(0.1, 1.33, 0);
		circle.setup(box2d.getWorld(), mouseX, mouseY, r);
		circles.push_back(circle);
	}
	if(key == 'z') {
		float r = ofRandom(3, 10);		// a random radius 4px - 20px
		CustomParticle p;
		p.setPhysics(0.4, 0.53, 0.31);
		p.setup(box2d.getWorld(), mouseX, mouseY, r);
		p.color.r = ofRandom(20, 100);
		p.color.g = 0;
		p.color.b = ofRandom(150, 255);
		customParticles.push_back(p);
	}	
	if(key == 'b') {
		float w = ofRandom(4, 20);	
		float h = ofRandom(4, 20);	
		ofxBox2dRect rect;
		rect.setPhysics(3.0, 0.53, 0.1);
		rect.setup(box2d.getWorld(), mouseX, mouseY, w, h);
		boxes.push_back(rect);
	}
	
	if(key == 's') bDrawLines = !bDrawLines;
	if(key == 'f') bMouseForce = !bMouseForce;
	if(key == 't') ofToggleFullscreen();
	
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	if(bDrawLines) {
		lineStrip.addPoint(x, y);
	}	
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	if(bDrawLines) {
		lineStrip.setWorld(box2d.getWorld());
		lineStrip.clear();
	}	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	if(bDrawLines) lineStrip.createShape();
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

