package xSocketServer;
 
import java.io.IOException;
import java.nio.BufferUnderflowException;
import java.nio.channels.ClosedChannelException;
import java.util.*;
import org.xsocket.*;
import org.xsocket.connection.*;
 
public class xSocketDataHandler implements IDataHandler, IConnectHandler, IDisconnectHandler
{
    private Set<INonBlockingConnection> sessions = Collections.synchronizedSet(new HashSet<INonBlockingConnection>());
 
    public boolean onData(INonBlockingConnection nbc) throws IOException, BufferUnderflowException, ClosedChannelException, MaxReadSizeExceededException
    {
        try
        {
            String data = nbc.readStringByDelimiter("\0");
            // Expected Message Format:
            // USERNAME~MESSAGE
            // ie: JohnDoe~Hello World
 
            if(data.trim().length() > 0)
            {
                //if (data.indexOf("TCP")==-1) {
                	// coming from flash
                	//System.out.println("Incoming data: " + data);
                //}
 
                if(data.equalsIgnoreCase("<policy-file-request/>"))
                {
                    nbc.write("<?xml version=\"1.0\"?><!DOCTYPE cross-domain-policy SYSTEM \"http://www.macromedia.com/xml/dtds/cross-domain-policy.dtd\"><cross-domain-policy><allow-access-from domain=\"*\" to-ports=\"*\"/></cross-domain-policy>\0");
                    return true;
                }
 
                // send messages
                if (data.indexOf("TCP")==-1) {
                	// coming from flash
                	// send cursor to oF
                	sendMessageToOF("f~" + data + "[/TCP]");
                	// send cursors to Flash
                    sendMessageToFlash("l~" + data + "\0");
                } else {
                	// coming from oF
                	// send messages to Flash
                    sendMessageToFlash("d~" + data + "\0");
                }
            }
        }
        catch(Exception ex)
        {
            System.out.println("onData: " + ex.getMessage());
        }
 
        return true;
    }
 
    private void sendMessageToOF(String message)
    {
        try
        {
            synchronized(sessions)
            {
                Iterator<INonBlockingConnection> iter = sessions.iterator();
 
                while(iter.hasNext())
                {
                    INonBlockingConnection nbConn = (INonBlockingConnection) iter.next();
 
                    if(nbConn.isOpen())
                        nbConn.write(message);
                }
            }
 
            //System.out.println("Outgoing data: " + message);
        }
        catch(Exception ex)
        {
            System.out.println("sendMessageToAll: " + ex.getMessage());
        }
    }
 
    private void sendMessageToFlash(String message)
    {
        try
        {
            synchronized(sessions)
            {
                Iterator<INonBlockingConnection> iter = sessions.iterator();
 
                while(iter.hasNext())
                {
                    INonBlockingConnection nbConn = (INonBlockingConnection) iter.next();
 
                    if(nbConn.isOpen())
                        nbConn.write(message);
                }
            }
 
            //System.out.println("Outgoing data: " + message);
        }
        catch(Exception ex)
        {
            System.out.println("sendMessageToAll: " + ex.getMessage());
        }
    }
 
    public boolean onConnect(INonBlockingConnection nbc) throws IOException, BufferUnderflowException, MaxReadSizeExceededException
    {
        try
        {
            synchronized(sessions)
            {
                sessions.add(nbc);
            }
 
            System.out.println("onConnect");
        }
        catch(Exception ex)
        {
            System.out.println("onConnect: " + ex.getMessage());
        }
 
        return true;
    }
 
    public boolean onDisconnect(INonBlockingConnection nbc) throws IOException
    {
        try
        {
            synchronized(sessions)
            {
                sessions.remove(nbc);
            }
 
            System.out.println("onDisconnect");
        }
        catch(Exception ex)
        {
            System.out.println("onDisconnect: " + ex.getMessage());
        }
 
        return true;
    }
}